package com.annimon.ownlang.lib;

import com.annimon.ownlang.exceptions.TypeException;
import com.annimon.stream.Iterable;
import com.annimon.stream.Objects;
import java.util.Enumeration;
import java.util.Hashtable;

/**
 *
 * @author aNNiMON
 */
public class MapValue implements Value, Iterable/*<Map.Entry<Value, Value>>*/ {
    
    public static final MapValue EMPTY = new MapValue(1);
    
    public static MapValue merge(MapValue map1, MapValue map2) {
        final MapValue result = new MapValue(map1.size() + map2.size());
        for (Enumeration en = map1.map.keys(); en.hasMoreElements(); ) {
            final Object key = en.nextElement();
            result.map.put(key, map1.map.get(key));
        }
        for (Enumeration en = map2.map.keys(); en.hasMoreElements(); ) {
            final Object key = en.nextElement();
            result.map.put(key, map2.map.get(key));
        }
        return result;
    }
    
    private final Hashtable/*<Value, Value>*/ map;

    public MapValue(int size) {
        this.map = new Hashtable/*<>*/(size);
    }

    public MapValue(Hashtable/*<Value, Value>*/ map) {
        this.map = map;
    }
    
    /*@Override*/
    public int type() {
        return Types.MAP;
    }
    
    public int size() {
        return map.size();
    }
    
    public boolean containsKey(Value key) {
        return map.containsKey(key);
    }

    public Value get(Value key) {
        return (Value) map.get(key);
    }
    
    public void set(Value key, Value value) {
        map.put(key, value);
    }
    
    /*@Override*/
    public Object raw() {
        return map;
    }
    
    /*@Override*/
    public int asInt() {
        throw new TypeException("Cannot cast map to integer");
    }

    /*@Override*/
    public double asNumber() {
        throw new TypeException("Cannot cast map to number");
    }

    /*@Override*/
    public String asString() {
        return map.toString();
    }

    /*@Override*/
    public Enumeration/*<Map.Entry<Value, Value>>*/ iterator() {
        return Objects.entrySet(map).elements();
    }

    /*@Override*/
    public int hashCode() {
        int hash = 5;
        hash = 37 * hash + Objects.hashCode(this.map);
        return hash;
    }

    /*@Override*/
    public boolean equals(Object obj) {
        if (this == obj) return true;
        if (obj == null) return false;
        if (getClass() != obj.getClass())
            return false;
        final MapValue other = (MapValue) obj;
        return Objects.equals(this.map, other.map);
    }
    
    /*@Override*/
    public int compareTo(Object obj) {
        if (!(obj instanceof Value)) return -1;
        
        final Value o = (Value) obj;
        if (o.type() == Types.MAP) {
            final int lengthCompare = Objects.compare(size(), ((MapValue) o).size());
            if (lengthCompare != 0) return lengthCompare;
        }
        return asString().compareTo(o.asString());
    }
    
    /*@Override*/
    public String toString() {
        return asString();
    }
}
