package com.annimon.ownlang.lib;

import com.annimon.stream.Objects;

/**
 *
 * @author aNNiMON
 */
public final class NumberValue implements Value {
    
    public static final NumberValue MINUS_ONE = new NumberValue(-1);
    public static final NumberValue ZERO = new NumberValue(0);
    public static final NumberValue ONE = new NumberValue(1);
    
    public static NumberValue fromBoolean(boolean b) {
        return b ? ONE : ZERO;
    }
    
    private final Object/*Number*/ value;
    
    public NumberValue(int value) {
        this.value = new Integer(value);
    }
    
    public NumberValue(long value) {
        this.value = new Long(value);
    }
    
    public NumberValue(double value) {
        this.value = new Double(value);
    }
    
    public NumberValue(Object/*Number*/ value) {
        this.value = value;
    }
    
    /*@Override*/
    public int type() {
        return Types.NUMBER;
    }
    
    /*@Override*/
    public Object/*Number*/ raw() {
        return value;
    }
    
    public boolean asBoolean() {
        return Objects.intValue(value) != 0;
    }
    
    public byte asByte() {
        return Objects.byteValue(value);
    }
    
    public short asShort() {
        return Objects.shortValue(value);
    }

    /*@Override*/
    public int asInt() {
        return Objects.intValue(value);
    }

    public long asLong() {
        return Objects.longValue(value);
    }

    public float asFloat() {
        return Objects.floatValue(value);
    }

    public double asDouble() {
        return Objects.doubleValue(value);
    }
    
    /*@Override*/
    public double asNumber() {
        return Objects.doubleValue(value);
    }

    /*@Override*/
    public String asString() {
        return value.toString();
    }

    /*@Override*/
    public int hashCode() {
        int hash = 3;
        hash = 71 * hash + value.hashCode();
        return hash;
    }

    /*@Override*/
    public boolean equals(Object obj) {
        if (this == obj) return true;
        if (obj == null) return false;
        if (getClass() != obj.getClass())
            return false;
        final Object/*Number*/ other = ((NumberValue) obj).value;
        if (value instanceof Double || other instanceof Double) {
            return Objects.compare(Objects.doubleValue(value), Objects.doubleValue(other)) == 0;
        }
        if (value instanceof Float || other instanceof Float) {
            return Objects.compare(Objects.floatValue(value), Objects.floatValue(other)) == 0;
        }
        if (value instanceof Long || other instanceof Long) {
            return Objects.compare(Objects.longValue(value), Objects.longValue(other)) == 0;
        }
        return Objects.compare(Objects.intValue(value), Objects.intValue(other)) == 0;
    }
    
    /*@Override*/
    public int compareTo(Object obj) {
        if (!(obj instanceof Value)) return -1;
        
        final Value o = (Value) obj;
        if (o.type() == Types.NUMBER) {
            final Object/*Number*/ other = ((NumberValue) o).value;
            if (value instanceof Double || other instanceof Double) {
                return Objects.compare(Objects.doubleValue(value), Objects.doubleValue(other));
            }
            if (value instanceof Float || other instanceof Float) {
                return Objects.compare(Objects.floatValue(value), Objects.floatValue(other));
            }
            if (value instanceof Long || other instanceof Long) {
                return Objects.compare(Objects.longValue(value), Objects.longValue(other));
            }
            return Objects.compare(Objects.intValue(value), Objects.intValue(other));
        }
        return asString().compareTo(o.asString());
    }

    /*@Override*/
    public String toString() {
        return asString();
    }
}
