package com.annimon.ownlang.parser.ast;

import com.annimon.ownlang.exceptions.OperationIsNotSupportedException;
import com.annimon.ownlang.lib.NumberValue;
import com.annimon.ownlang.lib.Types;
import com.annimon.ownlang.lib.Value;

/**
 *
 * @author aNNiMON
 */
public final class ConditionalExpression implements Expression {
    
    public static final int
        EQUALS = 1,//("=="),
        NOT_EQUALS = 2,//("!="),
        
        LT = 3,//("<"),
        LTEQ = 4,//("<="),
        GT = 5,//(">"),
        GTEQ = 6,//(">="),
        
        AND = 7,//("&&"),
        OR = 8,//("||");
        LAST = 8;
    
    private static final String[] OPERATION_NAMES = {
        "", "==", "!=", "<", "<=", ">", ">=", "&&", "||"
    };
    
    public static String operationName(int operation) {
        if (operation < 1 && operation > LAST) return "";
        return OPERATION_NAMES[operation];
    }
        
    public final Expression expr1, expr2;
    public final int operation;

    public ConditionalExpression(int operation, Expression expr1, Expression expr2) {
        this.operation = operation;
        this.expr1 = expr1;
        this.expr2 = expr2;
    }

    /*@Override*/
    public Value eval() {
        final Value value1 = expr1.eval();
        switch (operation) {
            case AND: return NumberValue.fromBoolean(
                    (value1.asInt() != 0) && (expr2.eval().asInt() != 0) );
            case OR: return NumberValue.fromBoolean(
                    (value1.asInt() != 0) || (expr2.eval().asInt() != 0) );
        }
        
        
        final Value value2 = expr2.eval();
        
        double number1, number2;
        if (value1.type() == Types.STRING) {
            number1 = value1.asString().compareTo(value2.asString());
            number2 = 0;
        } else {
            number1 = value1.asNumber();
            number2 = value2.asNumber();
        }
        
        boolean result;
        switch (operation) {
            case EQUALS: result = number1 == number2; break;
            case NOT_EQUALS: result = number1 != number2; break;
            
            case LT: result = number1 < number2; break;
            case LTEQ: result = number1 <= number2; break;
            case GT: result = number1 > number2; break;
            case GTEQ: result = number1 >= number2; break;
            
            default:
                throw new OperationIsNotSupportedException(operationName(operation));
        }
        return NumberValue.fromBoolean(result);
    }
    
    /*@Override*/
    public void accept(Visitor visitor) {
        visitor.visit(this);
    }

    /*@Override*/
    public String toString() {
        return expr1 + " " + operationName(operation) + " " + expr2;
    }
}
