package com.annimon.stream;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

/**
 * Common operations with Object.
 * @author aNNiMON
 */
public final class Objects {
    
    public static boolean equals(Object a, Object b) {
        return (a == b) || (a != null && a.equals(b));
    }
    
    public static int hashCode(Object o) {
        return o != null ? o.hashCode() : 0;
    }
    
    public static int hash(Object[] values) {
        if (values == null) return 0;
        
        int result = 1;
        for (int i = 0; i < values.length; i++) {
            result = 31 * result + hashCode(values[i]);
        }
        return result;
    }
    
    public static String toString(Object o, String nullDefault) {
        return (o != null) ? o.toString() : nullDefault;
    }
    
    public static int compare(Object a, Object b, Comparator c) {
        return (a == b) ? 0 : c.compare(a, b);
    }
    
    public static Object requireNonNull(Object obj) {
        if (obj == null)
            throw new NullPointerException();
        return obj;
    }
    
    public static Object requireNonNull(Object obj, String message) {
        if (obj == null)
            throw new NullPointerException(message);
        return obj;
    }
    
    public static int compare(int lhs, int rhs) {
        return lhs < rhs ? -1 : (lhs == rhs ? 0 : 1);
    }

    public static int compare(long lhs, long rhs) {
        return lhs < rhs ? -1 : (lhs == rhs ? 0 : 1);
    }
    
    public static int compare(float f1, float f2) {
        if (f1 < f2) return -1;
        if (f1 > f2) return 1;
        int thisBits    = Float.floatToIntBits(f1);
        int anotherBits = Float.floatToIntBits(f2);
        return (thisBits == anotherBits ?  0 :
                (thisBits < anotherBits ? -1 :
                 1));
    }
    
    public static int compare(double d1, double d2) {
        if (d1 < d2) return -1;
        if (d1 > d2) return 1;
        long thisBits    = Double.doubleToLongBits(d1);
        long anotherBits = Double.doubleToLongBits(d2);
        return (thisBits == anotherBits ?  0 :
                (thisBits < anotherBits ? -1 :
                 1));
    }
    
    public static double doubleValue(Object obj) {
        if (obj instanceof Double) return ((Double) obj).doubleValue();
        if (obj instanceof Float) return ((Float) obj).doubleValue();
        if (obj instanceof Long) return ((Long) obj).doubleValue();
        if (obj instanceof Integer) return ((Integer) obj).doubleValue();
        if (obj instanceof Short) return ((Short) obj).shortValue();
        if (obj instanceof Byte) return ((Byte) obj).byteValue();
        throw new ClassCastException("Unable to retrieve double value of " + obj.getClass());
    }
    
    public static float floatValue(Object obj) {
        if (obj instanceof Double) return ((Double) obj).floatValue();
        if (obj instanceof Float) return ((Float) obj).floatValue();
        if (obj instanceof Long) return ((Long) obj).floatValue();
        if (obj instanceof Integer) return ((Integer) obj).floatValue();
        if (obj instanceof Short) return ((Short) obj).shortValue();
        if (obj instanceof Byte) return ((Byte) obj).byteValue();
        throw new ClassCastException("Unable to retrieve float value of " + obj.getClass());
    }
    
    public static long longValue(Object obj) {
        if (obj instanceof Double) return ((Double) obj).longValue();
        if (obj instanceof Float) return ((Float) obj).longValue();
        if (obj instanceof Long) return ((Long) obj).longValue();
        if (obj instanceof Integer) return ((Integer) obj).longValue();
        if (obj instanceof Short) return ((Short) obj).shortValue();
        if (obj instanceof Byte) return ((Byte) obj).byteValue();
        throw new ClassCastException("Unable to retrieve long value of " + obj.getClass());
    }
    
    public static int intValue(Object obj) {
        if (obj instanceof Double) return ((Double) obj).intValue();
        if (obj instanceof Float) return ((Float) obj).intValue();
        if (obj instanceof Long) return (int)((Long) obj).longValue();
        if (obj instanceof Integer) return ((Integer) obj).intValue();
        if (obj instanceof Short) return ((Short) obj).shortValue();
        if (obj instanceof Byte) return ((Byte) obj).byteValue();
        throw new ClassCastException("Unable to retrieve int value of " + obj.getClass());
    }
    
    public static short shortValue(Object obj) {
        if (obj instanceof Double) return ((Double) obj).shortValue();
        if (obj instanceof Float) return ((Float) obj).shortValue();
        if (obj instanceof Long) return (short)((Long) obj).longValue();
        if (obj instanceof Integer) return ((Integer) obj).shortValue();
        if (obj instanceof Short) return ((Short) obj).shortValue();
        if (obj instanceof Byte) return ((Byte) obj).byteValue();
        throw new ClassCastException("Unable to retrieve short value of " + obj.getClass());
    }
    
    public static byte byteValue(Object obj) {
        if (obj instanceof Double) return ((Double) obj).byteValue();
        if (obj instanceof Float) return ((Float) obj).byteValue();
        if (obj instanceof Long) return (byte)((Long) obj).longValue();
        if (obj instanceof Integer) return ((Integer) obj).byteValue();
        if (obj instanceof Short) return (byte)((Short) obj).shortValue();
        if (obj instanceof Byte) return ((Byte) obj).byteValue();
        throw new ClassCastException("Unable to retrieve byte value of " + obj.getClass());
    }
    
    public static boolean deepEquals(Object[] a1, Object[] a2) {
        if (a1 == a2)
            return true;
        if (a1 == null || a2 == null)
            return false;
        int length = a1.length;
        if (a2.length != length)
            return false;

        for (int i = 0; i < length; i++) {
            Object e1 = a1[i];
            Object e2 = a2[i];

            if (e1 == e2)
                continue;
            if (e1 == null)
                return false;

            // Figure out whether the two elements are equal
            boolean eq;
            if (e1 instanceof Object[] && e2 instanceof Object[])
                eq = deepEquals((Object[]) e1, (Object[]) e2);
            else if (e1 instanceof byte[] && e2 instanceof byte[])
                eq = equals((byte[]) e1, (byte[]) e2);
            else if (e1 instanceof short[] && e2 instanceof short[])
                eq = equals((short[]) e1, (short[]) e2);
            else if (e1 instanceof int[] && e2 instanceof int[])
                eq = equals((int[]) e1, (int[]) e2);
            else if (e1 instanceof long[] && e2 instanceof long[])
                eq = equals((long[]) e1, (long[]) e2);
            else if (e1 instanceof char[] && e2 instanceof char[])
                eq = equals((char[]) e1, (char[]) e2);
            else if (e1 instanceof float[] && e2 instanceof float[])
                eq = equals((float[]) e1, (float[]) e2);
            else if (e1 instanceof double[] && e2 instanceof double[])
                eq = equals((double[]) e1, (double[]) e2);
            else if (e1 instanceof boolean[] && e2 instanceof boolean[])
                eq = equals((boolean[]) e1, (boolean[]) e2);
            else
                eq = e1.equals(e2);

            if (!eq)
                return false;
        }
        return true;
    }
    
    public static Vector asList(Object[] array) {
        final Vector v = new Vector(array.length);
        for (int i = 0; i < array.length; i++) {
            v.addElement(array[i]);
        }
        return v;
    }
    
    public static Vector/*<Entry>*/ entrySet(Hashtable map) {
        final Vector v = new Vector(map.size());
        for (Enumeration en = map.keys(); en.hasMoreElements(); ) {
            final Object key = en.nextElement();
            v.addElement(new Entry(key, map.get(key)));
        }
        return v;
    }
    
    public static boolean isLetterOrDigit(char ch) {
        return (ch >= 'a' && ch <= 'z') || 
               (ch >= 'A' && ch <= 'Z') || 
               (ch >= '0' && ch <= '9');
    }
    
    public static boolean isJavaIdentifierStart(char ch) {
        return (ch >= 'a' && ch <= 'z') || 
               (ch >= 'A' && ch <= 'Z') || 
                ch == '$' || ch == '_'  || 
               (ch >= 'а' && ch <= 'я') || 
               (ch >= 'А' && ch <= 'Я') || 
                ch == 'ё' || ch == 'Ё'  ;
    }
}
